#include "tree.def"
END_OF_BASE_TREE_CODES
#include "c-family/c-common.def"
#include "ada/gcc-interface/ada-tree.def"
#include "c/c-tree.def"
#include "cp/cp-tree.def"
#include "d/d-tree.def"
#include "m2/m2-tree.def"
#include "objc/objc-tree.def"
